import pytest
from orchestrator.types import SubscriptionLifecycle

from gso.products import ProductName
from gso.products.product_types.lan_switch_interconnect import ImportedLanSwitchInterconnect
from gso.utils.types.ip_address import AddressSpace
from test.workflows import (
    assert_complete,
    extract_state,
    run_workflow,
)


@pytest.fixture()
def workflow_input_data(faker, router_subscription_factory, switch_subscription_factory):
    return {
        "lan_switch_interconnect_description": faker.sentence(),
        "lan_switch_interconnect_ip_network": faker.ipv4_network(),
        "address_space": AddressSpace.PUBLIC,
        "minimum_links": 1,
        "router_side": {
            "node": router_subscription_factory(),
            "ae_iface": faker.nokia_lag_interface_name(),
            "ae_members": faker.link_members_nokia(),
            "ipv4_address": faker.ipv4(),
        },
        "switch_side": {
            "switch": switch_subscription_factory(),
            "ae_iface": faker.juniper_ae_interface_name(),
            "ae_members": faker.link_members_juniper(),
            "ipv4_address": faker.ipv4(),
        },
    }


@pytest.mark.workflow()
def test_create_imported_lan_switch_interconnect_success(workflow_input_data):
    result, _, _ = run_workflow("create_imported_lan_switch_interconnect", [workflow_input_data])
    state = extract_state(result)
    subscription = ImportedLanSwitchInterconnect.from_subscription(state["subscription_id"])

    assert_complete(result)
    assert subscription.product.name == ProductName.IMPORTED_LAN_SWITCH_INTERCONNECT
    assert subscription.status == SubscriptionLifecycle.ACTIVE
